'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.readDependencyTree = exports.extractModules = exports.CIRCULAR = undefined;

var _execFile = require('./exec-file');

var _execFile2 = _interopRequireDefault(_execFile);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const CIRCULAR = exports.CIRCULAR = '[Circular]';

const _extractLicenseName = license => {
  if (!license) return null;
  if (Array.isArray(license)) return license.map(_extractLicenseName);
  if (license.type || license.name) return _extractLicenseName(license.type || license.name);
  return license;
};

function* iterateDependencies(parent) {
  if (!parent.dependencies) return;

  for (const module of Object.values(parent.dependencies)) {
    if (!module.private) {
      const license = _extractLicenseName(module.license || module.licence || module.licenses || module.licences);

      yield {
        name: module.name,
        version: String(module.version),
        license
      };
    }
    yield* iterateDependencies(module);
  }
}

const extractModules = exports.extractModules = module => {
  const moduleMap = [];
  const circulars = {};
  for (const _module of iterateDependencies(module)) {
    const { name, version, license } = _module;
    const explicitName = `${name}@${version}`;
    if (license !== CIRCULAR) {
      moduleMap.push(_module);
      circulars[explicitName] = false;
    } else if (circulars[explicitName] === undefined) {
      circulars[explicitName] = true;
    }
  }
  for (const circular of Object.keys(circulars)) {
    if (circulars[circular]) {
      throw new Error(`${circular} has only circular references, cannot determine license`);
    }
  }
  return moduleMap;
};

const readDependencyTree = exports.readDependencyTree = async function (dev, cwd) {
  return (0, _execFile2.default)(cwd, 'npm', 'list', '--json', '--long', dev ? '--dev' : '--prod');
};

exports.default = async (dev, path) => extractModules((await readDependencyTree(dev, path)));